/*
 * This file is part of queso.
 *
 * Copyright © 2020 Canek Peláez Valdés
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *    Canek Peláez Valdés <canek@ciencias.unam.mx>
 */

namespace Queso {

    /**
     * Abstract class for audio and video devices.
     */
    public abstract class Device : GLib.Object {

        /**
         * Whether is a testing device.
         */
        public bool is_test { public get; protected set; }

        /**
         * The name of the device.
         */
        public string name { public get; protected set; }

        /* The GStreamer device. */
        private Gst.Device device;

        /**
         * Constructs a device.
         * @param device the GStreamer device.
         */
        public Device(Gst.Device device) {
            this.device = device;
            name = device.get_display_name();
            var caps = device.get_caps();
            if (caps == null)
                return;
            init_setups();
            for (uint i = 0; i < caps.get_size(); i++)
                parse_structure(caps.get_structure(i));
        }

        /**
         * Constructs a testing device.
         */
        public Device.test() {
            is_test = true;
            init_setups();
        }

        public Gst.Element create_element(string name) {
            return device.create_element(name);
        }

        /**
         * Initializes the device setups.
         */
        protected abstract void init_setups();

        /**
         * Parse an structure in the device.
         * @param structure the structure to parse.
         */
        protected abstract void parse_structure(Gst.Structure structure);
    }
}
