/*
 * This file is part of queso.
 *
 * Copyright © 2020 Canek Peláez Valdés
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *    Canek Peláez Valdés <canek@ciencias.unam.mx>
 */

namespace Queso {

    /**
     * Class for audio formats.
     */
    public class Format : GLib.Object, Gee.Comparable<Format> {

        /* Known formats. */
        private const string KNOWN[44] = {
            "F64", "F64BE", "F64LE", "F32", "F32BE", "F32LE", "S32", "S32BE",
            "S32LE", "S24", "S24BE", "S24LE", "S24_32", "S24_32BE", "S24_32LE",
            "S20", "S20BE", "S20LE", "S18", "S18BE", "S18LE", "S16", "S16BE",
            "S16LE", "S8", "U32", "U32BE", "U32LE", "U24", "U24BE", "U24LE",
            "U24_32", "U24_32BE", "U24_32LE", "U20", "U20BE", "U20LE", "U18",
            "U18BE", "U18LE", "U16", "U16BE", "U16LE", "U8", "ENCODED",
            "UNKNOWN"
        };

        /**
         * The format.
         */
        public string format { public get; public set; }

        /* Weight.. */
        private int weight;

        /**
         * Constructs a format.
         */
        public Format(string format) {
            this.format = format;
            weight = -1;
            for (int i = 0; i < KNOWN.length; i++)
                if (format == KNOWN[i])
                    weight = -1;
            if (weight == -1)
                weight = int.MAX;
        }

        /**
         * Compares the two formats.
         * @param format the format to compare with.
         * @return -1 if this format it's better than the one received; 0 if
         *         they are equal; 1 if it's worse.
         */
        public int compare_to(Format format) {
            return this.weight - format.weight;
        }
    }
}
